<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Movreal;

class Bco54200ExtratoBancario {

    //Mensagens padrões
    const CODREDU_NCAD = "Conta corrente não cadastrada.";
    const BANCO_INV = "Banco não encontrado no arquivo.";
    const ARQUIVO_INV = "Arquivo informado não é um arquivo de extrato bancário.";
    const EXTRATO_JA_IMP = "Os dados contidos no arquivo já existem no sistema.";

    //Classes
    protected $Bco54200factory;
    protected $environment;
    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $mask;
    protected $OfxService;

    //Variaveis para a importação de extrato;
    protected $cdcontcorr;
    protected $dscontcorr;
    protected $cdbanco;
    protected $cdreducont;
    protected $nrseqmovreal;
    protected $maskContcorr;
    protected $dataSet = array();
    protected $dataSetExtrato = array();

    public function __construct($Bco54200factory, $environment, $newCode, $entityManager, $mask, $OfxService) {
        $this->Bco54200factory = $Bco54200factory;
        $this->environment = $environment;
        $this->newCode = $newCode;
        $this->operador = $this->environment->getCdOperador();
        $this->nrorg = $this->environment->getNrOrg();
        $this->currentTime = $environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->entityManager = $entityManager;
        $this->mask = $mask;
        $this->OfxService = $OfxService;
    }

    /********************************** Extrato Bancário ***********************************/

    /**
     * Essa funçõ tem como objetivo calcular o saldo final do mês anterior ao mÊs da data da movimentação. Para isso
     * é buscado, à partir do ultimo dia do mês anterior à data da movimentação, o saldo inicial mais próximo. Depois
     * é calculado o saldo de todas as movimentações que existem para aquela conta corrente da data do saldo inicial
     * encontrado até o ultimo dia do mês anterior à data da movimentação. A diferença entre esses dois valores
     * encontrados será retornada.
     *
     * @param  string $dtMovreal  ultimo dia do mês anterior ao mês da data de movimentação
     * @param  string $cdreducont código reduzido da conta corrente
     * @return array              dados encontrados do banco
     */
    public function calcSaldFinMesAnt($dataFinal, $cdreducont) {
        //Busca o saldo inicial do extrato mais recente do sistema até o ultimo dia do mês anterior à data da movimentação
        $extratoSaldoIni = $this->Bco54200factory->getSaldoInicRecente($dataFinal, $cdreducont, $this->nrorg);
        //Calcula o saldo das movimentações, do dia do saldo inicial encontrado ao ultimo dia do mês anterior à data da movimentação
        $saldoPeriodo = $this->Bco54200factory->calculaSaldoPerio(substr($extratoSaldoIni['DTSALDINIC'], 0, 10), $dataFinal, $cdreducont, $this->nrorg);
        //Calcula o saldo final do mês anterior ao mês da data da movimentação (saldo inicial encontrado - saldo das movimentações do periodo)
        $saldoInicExtrato = $this->calculaValor($extratoSaldoIni['VRSALDINIC'], $extratoSaldoIni['IDEXTRINI']);


        $vrsaldfim = $saldoInicExtrato + $saldoPeriodo['SALDO'];

        if ($vrsaldfim < 0) {
            $idextrfim = 'D';
            $vrsaldfim = $vrsaldfim * (-1);
        } else {
            $idextrfim = 'C';
        }

        //Monta um array com os dados que serão usados por outro processo para salvar um extrato
        $abstratcExtrato = array ("IDEXTRFIM" => $idextrfim,
                                  "VRSALDFIM" => $vrsaldfim);

        return $abstratcExtrato;
    }

    /**
     * Essa função tem como finaliade chamar a função que vai excluir o registro da EXTRBANCO,
     * passando para ela os parametros certos.
     *
     * @param  array $row object com as informações da movimentação
     */
    public function deleteMovreal($row) {
        $this->Bco54200factory->removeMovReal($row['CDREDUCONT'], $row['CDSEQUEXTR'], $row['NRSEQMOVREAL'], $this->nrorg);
    }

    /**
     * Essa função é responsavel por gravar na tabela o log dessa operação. Porém ela não
     * esta sendo mais usada.
     *
     * @param array $row object com os dados da movimentação
     */
    public function geraLog($row) {
        $idativo = isset($row['IDATIVO']) ? ($row['IDATIVO'] ?: 'S') : 'S';
        $this->Bco54200factory->geraLog(
            $row['CDREDUCONT'], $row['CDSEQUEXTR'], $row['NRSEQMOVREAL'], $this->operador, $this->currentTime, $this->nrorg,
            $idativo, $this->currentTime, $this->nrorg, $this->operador, $this->currentTime, $this->nrorg, $this->operador
        );
    }

    /**
     * Essa função tem como objetivo retonar o primeiro e ultimo dia do mês anterior a data
     * que foi passada
     *
     * @param  string $data data que deseja pegar os dias do mes anterior
     * @return array        array assiativo com as duas datas
     */
    public function getDatasMesAnterior($data) {
        $mes = DateUtil::getMes($data);
        $ano = DateUtil::getAno($data);

        $mes  = intval($mes) - 1;
        if ($mes === 0) {
            $mes = '12';
            $ano = intval($ano) - 1;
        } else if ($mes < 10){
            $mes = '0'.$mes;
        }

        $primeiraData = '01/'.$mes.'/'.$ano;
        $ultimoDia = DateUtil::getUltimoDiaDoMes(DateUtil::getDataDeString($primeiraData))->format('d/m/Y');

        return array('primeiraDataMes'=>$primeiraData, 'ultimaDataMes'=>$ultimoDia);
    }

    /**
     * Essa função tem como objetivo verficar se existe um extrato no mês anterior à data da
     * movimentação a ser inserida ou editada e retorna-lo se existir. Se não existir verifica
     * se existem movimentações daquela conta corrente no banco. Se existir faz chama a função
     * faz o calculo do saldo final do mês anterior à data da movimentação. Se não existir
     * movimentações, considera o saldo final do mês anterior à data da movimentação como
     * 0.
     *
     * @param  string $dtMovreal  data da movimentação
     * @param  string $cdreducont código reduzido da conta corrente
     * @return array              dados do extrato salvo
     */
    public function  getExtratoMesAnt($dtMovreal, $cdreducont) {
        $datas = $this->getDatasMesAnterior($dtMovreal);
        $extrato = $this->Bco54200factory->getExtratoMes($datas['primeiraDataMes'], $datas['ultimaDataMes'], $cdreducont, $this->nrorg);

        if (empty($extrato)) {
            //Verifica as movimentações do banco de dados
            $movimentações = $this->Bco54200factory->registrosExtr($cdreducont, $this->nrorg);
            if ($movimentações['REGISTROS'] > 0) {
                //Se existir outros extratos, ajusta os registros no banco de dados
                $extrato = $this->calcSaldFinMesAnt($datas['ultimaDataMes'], $cdreducont);
            } else if ($movimentações['REGISTROS'] = 0) {
                //Se não existir insere um extrato para o mês anterior ao da movimentação
                $extrato = array ("IDEXTRFIM" => 'C',
                                  "VRSALDFIM" => 0);
            }
        }

        return $extrato;
    }

    /**
     * Essa função tem como finalidade receber o ultimo sequencial de movimentações do
     * extrato, e retornar o proimo sequencial valido. Para isso ele transformar o valor
     * em inteiro, somna com um. Depois converte o novo valor em string, calcula o numero
     * de caracteres existete para completar com 0 a esquerda até 10 caracteres.
     *
     * @param  string $sequencial sequencial de movimentação
     * @return string             sequencial formatado
     */
    public function getSequencialFormated($sequencial) {
        $sequencial = intval($sequencial);
        $sequencial = $sequencial + 1;
        $sequencial = strval($sequencial);
        $sequencial = str_pad($sequencial, 10, "0", STR_PAD_LEFT);
        return $sequencial;
    }

    /**
     * Essa função tem como objetivo salvar um registro natabela EXTRBANCO, gerando o sequencial
     * da tabela e dando o insert. É retornado os principais dados do registro salvo.
     *
     * @param  string $dataInic   data do saldo incial
     * @param  string $datafin    data do saldo final
     * @param  string $saldoIni   valor do saldo inicial
     * @param  string $saldoFin   valor do saldo final
     * @param  string $idSaldoIni identificador de debito ou credo do saldo incial
     * @param  string $idSaldoFin identificador de debito ou credo do saldo final
     * @param  string $cdreducont código reduzido da conta corrente
     * @return array              principais informações do registro salvo
     */
    public function insertExtratoSemExtratoAnt($dataInic, $datafin, $saldoIni, $saldoFin, $idSaldoIni, $idSaldoFin, $cdreducont) {
        //Gera o sequencial da tabela EXTRBANCO
        $cdseqextr = $this->newCode->getCodeByTableName('MOVREAL'.$cdreducont, $this->nrorg);

        $this->Bco54200factory->insertExtrBanco(
            $cdreducont, $cdseqextr, $dataInic, $datafin, $saldoIni, $saldoFin,
            $idSaldoIni, $idSaldoFin, $this->nrorg, 'S', $this->currentTime, $this->nrorg,
            $this->operador, $this->currentTime, $this->nrorg, $this->operador
        );

        $extratoSalvo = array('CDREDUCONT' => $cdreducont,
                              'CDSEQUEXTR' => $cdseqextr,
                              'DATAINIC' => $dataInic,
                              'DATAFIM' => $datafin,
                              'SALDOINI' => $saldoIni,
                              'SALDOFIM' => $saldoFin,
                              'IDSALDOINI' => $idSaldoIni,
                              'IDSALDOFIM' => $idSaldoFin);

        return $extratoSalvo;
    }

    /**
     * Essa função tem como finalidade formatar alguns campos antes de inserir um registro
     * na tabela MOVREAL, ou seja, inserir uma movimentação.
     *
     * @param  array $row object com os dados da movimentação
     */
    public function insertMovReal($row) {
        try{
            
            if (!array_key_exists('IDTPMOVREAL', $row)) {
                $row['IDTPMOVREAL'] = 1;
            }
            //$dshistmovreal = substr($row['DSHISTMOVREAL'], 0, 40);
            $dshistmovreal = $row['DSHISTMOVREAL'];
            $cdfitidextr = isset($row['CDFITIDEXTR'])?$row['CDFITIDEXTR']:null;
            
            // Verifica se o $cdfitidextr é maior que 50 caracteres, se for ele corta a string deixando abreviado.
            if(strlen($cdfitidextr) > 50) {
                $cdfitidextr = substr($cdfitidextr, 0, 50);
            }
            
            $this->Bco54200factory->insertMovReal(
                $row['CDREDUCONT'], $row['CDSEQUEXTR'], $row['NRSEQMOVREAL'], $row['NRDOCMOVREAL'], $row['DTMOVREAL'], $row['VRMOVREAL'],
                $row['IDMOVREAL'], $dshistmovreal, '0', '1', '0000', null, $row['IDTPMOVREAL'], $this->nrorg,
                'S', $this->currentTime, $this->nrorg, $this->operador, $this->currentTime, $this->nrorg, $this->operador, $cdfitidextr
            );
        }catch(\Exceptin $e){
            throw $e;
        }
    }

    /**
     * Essa função tem como objetivo verirficar se existem movimentações no periodo do extrato da
     * movimentação sendo modificada, e com isso recalcular o saldo final, se ainda existir
     * movimentações no periodo ou excluir o extrato, caso não haja mais movimentações daquele
     * periodo.
     *
     * @param  array $row dados da movimentação que sendo modificada
     */
    public function recalculoExtrato($row) {
        $dtMovreal = DateUtil::getDataDeString(substr($row['DTMOVREAL'], 0, 10));
        $primeiroDia = DateUtil::getPrimeiroDiaDoMes($dtMovreal)->format('d/m/Y');
        $ultimoDia = DateUtil::getUltimoDiaDoMes($dtMovreal)->format('d/m/Y');
        //Busca as movimentações do periodo do extato
        $numeroDeMovimentacoes = $this->Bco54200factory->getMovsPeriodo($primeiroDia, $ultimoDia, $row['CDREDUCONT'], $this->nrorg);

        if ($numeroDeMovimentacoes['LINHAS'] > 0) {
            //Busca o saldo das movimentações
            $saldo = $this->Bco54200factory->calculaSaldoPerio($primeiroDia, $ultimoDia, $row['CDREDUCONT'], $this->nrorg);

            //Busca o extrato daquele periodo
            $extrato = $this->Bco54200factory->getExtratoMes($primeiroDia, $ultimoDia, $row['CDREDUCONT'], $this->nrorg);
            
            if(empty($extrato)){
                $extrato = $this->Bco54200factory->getExtratoCdsequextr($row['CDSEQUEXTR'], $row['CDREDUCONT'], $this->nrorg);
            }
            
            //Calcula o novo valor final do extrato
            $extrato['VRSALDINIC'] = $this->calculaValor($extrato['VRSALDINIC'], $extrato['IDEXTRINI']);

            $extrato['VRSALDFIM'] = $extrato['VRSALDINIC'] + $saldo['SALDO'];

            if ($extrato['VRSALDFIM'] < 0) {
                $extrato['IDEXTRFIM'] = 'D';
                $extrato['VRSALDFIM'] = $extrato['VRSALDFIM'] * (-1);
            } else {
                $extrato['IDEXTRFIM'] = 'C';
            }

            //Updata o extrato
            $this->Bco54200factory->updateExtrBanco($row['CDREDUCONT'], $extrato['CDSEQUEXTR'], $extrato['VRSALDFIM'],
                                                    $extrato['IDEXTRFIM'], $this->nrorg, $this->currentTime, $this->nrorg, $this->operador);
        } else {
            //Exclui o extrato
            $this->Bco54200factory->removeExtrBanco($row['CDREDUCONT'], $row['CDSEQUEXTR'], $this->nrorg);
        }
    }

    /**
     * Essa função tem como objetivo dar um update em um unico registro da tabela MOVREAL,
     * ela é usada quando o usário vai editar uma movimnetação bancária.
     *
     * @param  array $row dados da movimentação
     */
    public function updateMovReal($row) {
        //$dshistmovreal = substr($row['DSHISTMOVREAL'], 0, 40);
        $dshistmovreal = $row['DSHISTMOVREAL'];
        $cdfitidextr = isset($row['CDFITIDEXTR'])?$row['CDFITIDEXTR']:null;
        $this->Bco54200factory->updateMovReal(
            $row['CDREDUCONT'], $row['CDSEQUEXTR'], $row['NRSEQMOVREAL'], $row['NRDOCMOVREAL'],
            $row['DTMOVREAL'], $row['VRMOVREAL'], $row['IDMOVREAL'], $dshistmovreal,
            $row['IDSTCONMVBAN'], $row['IDTIPOREGI'], $row['CDCATELANC'], $row['IDCONCREAL'],
            $row['IDTPMOVREAL'], $this->nrorg, 'S', $this->currentTime, $this->nrorg,
            $this->operador, $cdfitidextr
        );
    }

    /**
     * Essa função tem como finalidade inserir os campos sequenciais da movimentação para que
     * ela seja salva. Para isso é preciso verificar a existencia de um extrato para o mês da
     * movimentação a ser inserida ou editada e fazer as validações cabiveis ao extrato.
     *
     * @param  array $row dados da movimentação a ser salvos
     * @return array      dados da movimentação a ser salvos com as colunas inseridas
     */
    public function validaExtrato($row) {
        //Verifica se existe extrato de um mês, ao mês anterior a data de movimentação
        if ($row['__is_new']) {
            //Pega o primero e ultimo dia do mes para buscar o extrato do mes da movimentação
            $dtMovreal = DateUtil::getDataDeString($row['DTMOVREAL']);
            $primeiroDia = DateUtil::getPrimeiroDiaDoMes($dtMovreal)->format('d/m/Y');
            $ultimoDia = DateUtil::getUltimoDiaDoMes($dtMovreal)->format('d/m/Y');

            //Verifica a existencia de um extrato para o mês da movimentação
            $extratoCorrente = $this->Bco54200factory->getExtratoMes($primeiroDia, $ultimoDia, $row['CDREDUCONT'], $this->nrorg);
            if (empty($extratoCorrente)) {
                $extratoAnterior = $this->getExtratoMesAnt($row['DTMOVREAL'], $row['CDREDUCONT']);
                $extratoCorrente = $this->insertExtratoSemExtratoAnt($primeiroDia, $ultimoDia, $extratoAnterior['VRSALDFIM'], 0, $extratoAnterior['IDEXTRFIM'], 'C', $row['CDREDUCONT']);
            }

            //Pega o sequencial do extrato para fazer a ligação com a tabela EXTRBANCO
            $row['CDSEQUEXTR'] = $extratoCorrente['CDSEQUEXTR'];
            
            //Busca o maior sequencial das movimentações daquele extrato
            $sequencial = $this->Bco54200factory->getNrseqmovreal($row['CDREDUCONT'], $row['CDSEQUEXTR'], $this->nrorg);
            
            if (!empty($sequencial['NRSEQMOVREAL'])) {
                $row['NRSEQMOVREAL'] = $this->getSequencialFormated($sequencial['NRSEQMOVREAL']);
            } else {
                $row['NRSEQMOVREAL'] = '0000000001';
            }
        }
        return $row;
    }

    /**
     * Essa função tem como objetivo verificar se operador que está logado tem permissão
     * para realizar a operação que ele está tentando realizar.
     *
     * @param  string $cdreducont código reduzido da conta corrente
     */
    public function validaOperador($cdreducont) {
        $validacao = $this->Bco54200factory->validaOperador($this->operador, $cdreducont, $this->nrorg);
        if(empty($validacao)){
            throw new \Exception("Operador não relacionado a empresa da conta corrente informada.", 1);
        }
    }

    /***************************** Importação de Extrato Bancário ****************************/

    /**
     * Essa função tem como finalidade contar quatos caracteres tem a mascara da
     * conta corrente, excluindo os caracteres separadores (- e .), para isso
     * ela busca no banco qual a mascara de conta corrente daquele banco. Além
     * disso ela grava na variavel global a máscara formatada para que ela possa
     * ser usada posterior- mente.
     *
     * @param  strung $cdbanco código do banco que ele deve pegar máscara
     * @return int             numero de caracteres da mascara que não são traço e ponto
     */
    public function calcNumberMask($cdbanco) {
        //Busca a mascara de conta corrente daquele banco
        $bank = $this->Bco54200factory->getCdpictcont($cdbanco, $this->nrorg);
        //Formata a mascara para que seja possivel aplicala usando a classe Mask
        $this->maskContcorr = str_replace('9', '#', $bank['CDPICTCONT']);

        $caracter = 0;
        for($i = 0; $i < strlen($this->maskContcorr); $i++) {
            if(substr($this->maskContcorr, $i, 1) !== '-' && substr($this->maskContcorr, $i, 1) !== '.') {
                $caracter++;
            }
        }
        return $caracter;
    }

    /**
     * Essa função tem como finalidade receber a mascara da conta corrente e verificar se ha
     * digito de verificação, e quantos digitos de verificação existem na mascara.
     *
     * @param  string $mask mascara da conta corrente
     * @return int          numero de digitos de identificação da conta
     */
    public function calcNumberIndent($mask) {
        $verificador = explode('-', $mask);
        if($verificador[1]) {
            return strlen($verificador[1]);
        } else {
            return 0;
        }
    }

    /**
     * Essa função tem como finalidade deixar o numero negativo quando o valor da operação
     * foi de debito (D).
     * @param  float $valor    valor a ser convertido ou não
     * @param  string $operacao indentificador de crédito ou débito
     * @return float           valor aplicado a multiplicação conforme o identificador
     */
    public function calculaValor($valor, $operacao) {
        if($operacao === 'D' && $valor > 0) {
            return $valor * (-1);
        } else {
            return $valor;
        }
    }

    /**
     * Essa função tem como finalidade receber uma string codificada em base 64 e
     * descidificala para ser usada nas funções dessa classe.
     *
     * @param  string $encodedFile codificada em base64
     * @return string              decodificada
     */
    public function decodeFile($encodedFile) {
        $file = base64_decode($encodedFile);
        return $file;
    }

    /**
     * Essa função tem como finalidade adcionar no array de movimentações do extrato, a
     * movimentação que foi lida na linha do arquivo e que ainda não está no sistema.
     *
     * @param  float $vrLanc         valor da movimentação
     * @param  string $operacao      indetificador de crédito e débito
     * @param  string $dtMovreal     data da movimentação
     * @param  string $nrdocmovreal  número do documento da movimentação
     * @param  string $dshistmovreal descrição da movimentação
     * @param  string $cdcatelanc
     */
    public function lanc($vrLanc, $operacao, $dtMovreal, $nrdocmovreal, $dshistmovreal, $cdcatelanc) {
        $lancamento = $this->Bco54200factory->getPesqLanc($this->cdreducont, $nrdocmovreal, $dtMovreal,
                                            round($vrLanc, 2), $this->nrorg);

        if(empty($lancamento)) {
            $saveRow = 'S';
        } else {
            $saveRow = 'N';
        }

        $this->dataSet[] = array (
            "VRMOVREAL" => $vrLanc,
            "IDMOVREAL" => $operacao,
            "DTMOVREAL" => $dtMovreal,
            "DSHISTMOVREAL" => $dshistmovreal,
            "CDREDUCONT" => $this->cdreducont,
            "NRDOCMOVREAL" => $nrdocmovreal,
            "IDTIPOREGI" => 3,
            "CDCATELANC" => $cdcatelanc,
            "IDSTCONMVBAN" => 0,
            "IDTPMOVREAL" => 0,
            "NRORG" => $this->nrorg,
            "SAVE" => $saveRow,
            "__is_new" => true
        );
    }

    /**
     * COMPLETAR DOCUMENTAÇÃO
     * @param  [type] $encodedFile [description]
     * @return [type]              [description]
     */
    public function readFileOfx($encodedFile, $empresa, $nrorg_access = null, $accesStage = false) {
        $file = $this->decodeFile($encodedFile);
        $extrato = $this->OfxService->readOfx($file);
        
        $movimentacoes = $extrato['MOVS'];
        array_pop($extrato);

        if($this->nrorg == NULL){
            $this->nrorg = $nrorg_access;
            $accesStage = true;
        }
        
        $cdbanccnab = substr($extrato['BANCO'], strlen($extrato['BANCO']) - 3);
        $numberCarater = $this->calcNumberMask($cdbanccnab);
        $cdcontcorr = trim($extrato['CONTA']);
        
        if ($cdbanccnab === '001') {
            $contas = $this->Bco54200factory->getContasBrasil($cdcontcorr, $cdbanccnab, $this->nrorg);
            if($contas) {
                $cdcontcorr = $contas[0]["CDCONTCORR"];
            }
        } else {
            $contas = $this->Bco54200factory->getContas($cdcontcorr, $cdbanccnab, $this->nrorg, $empresa);
            if($contas) {
                $cdcontcorr = $contas[0]["CDCONTCORR"];
            }
        }
        
        if (count($contas) === 0) {
            $banco = $this->Bco54200factory->getBancoByCdccnab($cdbanccnab);
            if (!empty($banco)) {
                /*
                Validação para não impedir que o erro na leitura do arquivo OFX por falta de cadastro não interrompa todo o processo
                do AccesStage
                */
                if($accesStage){
                    //throw new \Exception(self::CODREDU_NCAD . "<br>Banco: " . trim($banco[0]['NMBANCO']) . "<br>Número da conta: " . $cdcontcorr, 1);
                    return array('Error' => (self::CODREDU_NCAD . "<br>Banco: " . trim($banco[0]['NMBANCO']) . "<br>Número da conta: " . $cdcontcorr));
                } else {
                    throw new \Exception(self::CODREDU_NCAD . "<br>Banco: " . trim($banco[0]['NMBANCO']) . "<br>Número da conta: " . $cdcontcorr, 1);
                    
                }
            } else {
                /*
                Validação para não impedir que o erro na leitura do arquivo OFX por falta de cadastro não interrompa todo o processo
                do AccesStage
                */
                if($accesStage){
                    //throw new \Exception(self::CODREDU_NCAD . "<br>Número da conta: " . $cdcontcorr, 1);
                    return array('Error' => (self::CODREDU_NCAD . "<br>Número da conta: " . $cdcontcorr));
                } else {
                    throw new \Exception(self::CODREDU_NCAD . "<br>Número da conta: " . $cdcontcorr, 1);
                }
            }

        } else if (count($contas) === 1) {
            $cdreducont = $contas[0]['CDREDUCONT'];
        } else {
            $cdreducont = array();
            $tpcontas = array();
            for ($i=0; $i < count($contas) ; $i++) {
                $cdreducont[$contas[$i]['IDTIPOCCORR']] = $contas[$i]['CDREDUCONT'];
                // $tpcontas[] = $contas[$i]['IDTIPOCCORR'];
                $tpcontas[] = [
                    $contas[$i]['DSCONTCORR'],
                    $contas[$i]['IDTIPOCCORR']
                ];
            }
            $extrato['TPCONTAS'] = $tpcontas;
        }

        for ($i=0; $i < count($movimentacoes) ; $i++) {
            $movimentacoes[$i]['CDREDUCONT'] = $cdreducont;
            $movimentacoes[$i]['NRORG'] = $this->nrorg;
            $movimentacoes[$i]['SAVE'] = 'S';
            $movimentacoes[$i]['__is_new'] = true;
        }

        $extrato['NMBANCO'] = $contas[0]['NMBANCO'];
        $extrato['SALDOINICIAL'] = $this->calculaValor($extrato['VRSALDINIC'], $extrato['IDEXTRINI']);
        $extrato['SALDOFINAL'] = $this->calculaValor($extrato['VRSALDFIM'], $extrato['IDEXTRFIM']);
        $extrato['CDCONTCORRMASK'] = $this->mask->mask($cdcontcorr, $this->maskContcorr);
        $extrato['CONTAS'] = count($contas);

        return array($extrato, $movimentacoes);
    }

    /**
     * COMPLETAR DOCUMENTAÇÃO
     * Essa função tm como objetivo chamar a função de decodificar a string com o conteudo
     * do arquivo e, conforme o banco passado, chamar a rotina para ler o arquivo e montar
     * as informações necessárias para o salcamento.
     *
     * @param  string $encodedFile todo o conteudo do arquivo em base 64
     * @param  string $banco       CDBANCCNAB do banco
     */
    public function  readFileTxt($encodedFile, $banco) {
        $file = $this->decodeFile($encodedFile);
        if($banco === '237') { //Seleciona leitura layout Bradesco
            $this->readBradesco($file, $banco);
        } else if($banco === '033') { //Seleciona leitura layout Santander
            $this->readSantander($file, $banco);
        } else if($banco === '001') { //Seleciona leitura layout Brasil | Nossa Caixa | HSBC
            $this->readFebraban($file, $banco);
        } else if($banco === '341') { //Seleciona leitura layout Itau
            $this->selectProcedureRead($file, $banco);
        } else {
            throw new \Exception("Não existe suporte para importação de extrato para o banco informado.", 1);
        }

        if ( $this->verificaMovSerSalva() ) {
            return array($this->dataSetExtrato, $this->dataSet);
        } else {
            throw new \Exception(self::EXTRATO_JA_IMP, 1);
        }
    }

    /**
     * Essa função tem como finalidade receber uma string que contém quebras de linhas,
     * e retornar em um array as linhas, onde cada posição do array contém uma linha da
     * string lida.
     *
     * @param $file | string completa com quebras de linha
     *
     * @return $file | array de strings, onde cada posição é uma linha
     */
    public function returnStringInlines($file) {
        $file = preg_split('/\R/', $file);
        return $file;
    }

    /**
     * Essa função tem como finalidade setar o array que coresponde as informações
     * do extrato bancário sendo importado, quando se está lendo a linha de saldo
     * inicial do extrato. Nessa função é colocado o saldo inicial e a data incial
     * do extrato.
     *
     * @param  float $vrLanc    valor da movimentação
     * @param  string $operacao  identificador de crédito e débito
     * @param  string $dtMovreal data da movimentação
     */
    public function saldoIni($vrLanc, $operacao, $dtMovreal) {
        $this->dataSetExtrato['DTSALDINIC'] = $dtMovreal;
        $this->dataSetExtrato['SALDOINICIAL'] = $this->calculaValor($vrLanc, $operacao);
    }

    /**
     * Essa função tem como finalidade setar o arrayy que corres´ponde as informações
     * do extrato bancário sendo importando, quando se está lendo a linha do saldo final
     * do extrato. Nessa função é colocado o saldo final, a data final do extrato e a
     * conta corrente.
     *
     * @param  float $vrLanc    valor da movimentação
     * @param  string $operacao  identificador de crédito e débito
     * @param  string $dtMovreal data da movimentação
     */
    public function saldoFim($vrLanc, $operacao, $dtMovreal) {
        $this->dataSetExtrato['DTSALDFIN'] = $dtMovreal;
        $this->dataSetExtrato['SALDOFINAL'] = $this->calculaValor($vrLanc, $operacao);
        $this->dataSetExtrato['CDCONTCORRMASK'] = $this->mask->mask($this->cdcontcorr, str_replace('9', '#', $this->maskContcorr));
    }

    /**
     * Essa função tem como objetivo verificar quantos caraceres existem na linha do aquivo
     * para que a rotina de leitura correta seja chamada.
     *
     * @param  string $file  arquivo para fazer a verificação
     * @param  string $banco CDBANCCNAB do banco
     */
    public function selectProcedureRead($file, $banco) {
        $linesFile = $this->returnStringInlines($file);
        if(count($linesFile[0]) === 200) {
            //Chama a torina que le o arquvio de 200 caracteres
            $this->readItau200($file, $banco);
        } else if (count($linesFile[0]) === 240) {
            //Chama a rotina que le o arquivo de 240 caracteres
            $this->readItau240($file, $banco);
        }
    }

    /**
     * Essa função tem como objetivo verificar se existem movimentações a serem salvas
     * depois que o arquivo foi lindo. Pode ser que o extrato já tenha sido importado,
     * então nenhuma movimentação vai ser salva.
     *
     * @return boolean true para se existem registros a serem salvos, false para não
     */
    public function verificaMovSerSalva() {
        foreach ($this->dataSet as $movimentacao) {
            if ($movimentacao['SAVE'] === 'S') {
                return true;
            }
        }
        return false;
    }

    /******************************** Funções de leitura de arquvio *************************/

    /**
     * Importação de extratos de conta corrente do Banco Bradesco
     *
     * @param  string $file  dados do extrato
     * @param  string $banco CDBANCCNAB do banco
     */
    public function readBradesco($file, $banco) {
        $fileLines = $this->returnStringInlines($file);
        foreach ($fileLines as $line) {
            if(trim(substr($line, 0, 1)) === '1') {
                //Gravando conta corrente
                if(trim(substr($line,40, 1)) === 'P') {
                   $this->cdcontcorr = trim(substr($line, 31, 9)).'0';
                } else {
                   $this->cdcontcorr = trim(substr($line, 30, 10));
                }

                if(empty($this->cdbanco)) {
                    //Calcula o número de dígitos que compõe a conta corrente
                    $numberCarater = $this->calcNumberMask($banco);
                    //Formata o CDCONTCOR para buscar a CDREDUCONT
                    $this->cdcontcorr = str_pad(trim($this->cdcontcorr), $numberCarater, '0', STR_PAD_RIGHT);

                    $cdagencia = trim(substr($line, 17, 4));
                    //Busca o CDREDUCONT
                    $cdreducont = $this->Bco54200factory->getContCorrLike($banco, $cdagencia,
                                                                      $this->cdreducont,
                                                                      $this->nrorg);
                    if(empty($cdreducont)) {
                        throw new \Exception(self::CODREDU_NCAD . "<br>Agência: " . $cdagencia . "<br>Número da conta corrente: " . $this->cdcontcorr, 1);
                    } else {
                        $this->cdreducont = $cdreducont['CDREDUCONT'];
                    }
                }

                $this->cdbanco = trim(substr($line, 191, 3));

                //O select executado por essa função tras mais do que a mascara
                $infoBancos = $this->Bco54200factory->getCdpictcont($banco, $this->nrorg);

                if(empty($infoBancos) || $infoBancos['CDBANCCNAB']) {
                    throw new Exception(self::BANCO_INV, 1);

                }
                //Monta a data e movimentação contida na linha
                $dia = trim(substr($line, 80, 2));
                $mes = trim(substr($line, 82, 2));
                $ano = '20'.trim(substr($line, 84, 2));
                $dtMovreal = $dia.'/'.$mes.'/'.$ano;
                //Verifica qual a operação contida na linha
                $tipo = trim(substr($line, 41, 1)); //[0-Saldo Ini] [1-Lanc] [2-Saldo Fim]
                //Verifica qual o valor da operação contida na linha
                $vrLanc = floatval(trim(substr($line, 86, 16)).'.'.trim(substr($line, 102, 2)));
                //Verifica se é debito ou credito
                $operacao = trim(substr($line, 104, 1)); // D-Debito / C-Credito

                if($tipo === '0') {
                    $this->saldoIni($vrLanc, $operacao, $dtMovreal);
                } else if ($tipo === '1') {
                    $nrdocmovreal = str_pad(trim(substr($line, 74, 6)), 20, '0', STR_PAD_LEFT);
                    $dshistmovreal = trim(substr($line, 49, 25));
                    $cdcatelanc = trim(substr($line, 42, 3));
                    $this->lanc($vrLanc, $operacao, $dtMovreal);
                } else if ($tipo === '2') {
                    $this->saldoFim($vrLanc, $operacao, $dtMovreal);
                }
            }
        };
    }

    /**
     * Importação de extratos de conta corrente do Banco Itau para o aquivo de tamanho
     * de linha de 200.
     *
     * @param  string $file  dados do extrato
     * @param  string $banco CDBANCCNAB do banco
     */
    public function readItau200($file, $banco) {
        $fileLines = $this->returnStringInlines($file);
        foreach($fileLines as $line) {
            if(trim(substr($line, 0, 1)) === '1') {
                if(empty($this->cdbanco)) {
                    //Calcula o número de dígitos que compõe a conta corrente
                    $numberCarater = $this->calcNumberMask($banco);
                    //Formata o CDCONTCOR para buscar a CDREDUCONT
                    $this->cdcontcorr = str_pad($this->cdcontcorr, $numberCarater, '0', STR_PAD_RIGHT);

                    $requisicao = $this->Bco54200factory->getContCorrLike($banco, trim(substr($line, 27, 4)), $this->cdcontcorr, $this->nrorg);

                    if (empty($requisicao)) {
                        $agencia = trim(substr($line, 27, 4));
                        throw new \Exception(self::CODREDU_NCAD . "<br>Agência: " . $agencia . "<br>Número da conta corrente: " . $this->cdcontcorr, 1);
                    } else {
                        $this->cdreducont = $requisicao['CDREDUCONT'];
                        $this->dscontcorr = $requisicao['DSCONTCORR'];
                    }
                }

                if($banco === trim(substr($line, 191, 3))) {
                    //Seta cdbanco a inserir o extrato
                    $this->cdbanco = trim(substr($line, 191, 3));
                } else {
                    throw new \Exception(self::BANCO_INV, 1);
                }

                //Monta a data do lançamento
                $dia = trim(substr($line, 181, 2));
                $mes = trim(substr($line, 183, 2));
                $ano = trim(substr($line, 185, 4));
                $dtMovreal = $dia.'/'.$mes.'/'.$ano;

                //Tipo
                $tipo = trim(substr($line, 41, 1)); //[0-Saldo Ini] [1-Lanc] [2-Saldo Fim]

                //Valor
                $vrLanc = floatval(trim(substr($line, 86, 16)).'.'.trim(substr($line, 102, 2)));

                //Credito ou debito
                $operacao = trim(substr($line, 104, 1)); // D-Debito / C-Credito

                if (!empty($operacao)){
                    if($tipo === '0') {
                        $this->saldoIni($vrLanc, $operacao, $dtMovreal);
                    } else if ($tipo === '1') {
                        $nrdocmovreal = str_pad(trim(substr($line, 74, 6)), 20, '0', STR_PAD_LEFT);
                        $dshistmovreal = trim(substr($line, 49, 25));
                        $cdcatelanc = trim(substr($line, 42, 3));
                        $this->lanc($vrLanc, $operacao, $dtMovreal, $nrdocmovreal,
                                    $dshistmovreal, $cdcatelanc);
                    } else if ($tipo === '2') {
                        $this->saldoFim($vrLanc, $operacao, $dtMovreal);
                    }
                }
            } else {
                $this->cdbanco = '';
            }
        }
    }

    /**
     * Importação de extratos de conta corrente do Banco Itau para o aquivo de tamanho
     * de linha de 240.
     *
     * @param  string $file  dados do extrato
     * @param  string $banco CDBANCCNAB do banco
     */
    public function readItau240($file, $banco) {
        $fileLines = $this->returnStringInlines($file);
        foreach($fileLines as $line) {
            //Desconsidera header e trailer de arquivo
            $checkLinha1 = trim(substr($line, 7, 1));
            $checkLinha2 = trim(substr($line, 7, 1));
            if(($checkLinha1 !== '0' && $checkLinha1 !== "") &&  ($checkLinha2 !== '9' && $checkLinha2 !== "")) {

                //Verifica se o arquivo é um arquivo de extrato para conciliação
                if(trim(substr($line, 7, 1)) === '1' && trim(substr($line, 8, 1)) === 'E') {
                    throw new \Exception(self::ARQUIVO_INV, 1);
                }

                //Executa apenas uma vez
                if(empty($this->cdbanco)) {
                    //Calcula o número de dígitos que compõe a conta corrente
                    $numberCarater = $this->calcNumberMask($banco);
                    //Formata o CDCONTCOR para buscar a CDREDUCONT
                    $this->cdcontcorr = str_pad($this->cdcontcorr, $numberCarater, '0', STR_PAD_RIGHT);

                    $requisicao = $this->Bco54200factory->getContCorrLike($banco, trim(substr($line, 27, 4)), $this->cdcontcorr, $this->nrorg);

                    if (empty($requisicao)) {
                        $agencia = trim(substr($line, 27, 4));
                        throw new \Exception(self::CODREDU_NCAD . "<br>Agência: " . $agencia . "<br>Número da conta corrente: " . $this->cdcontcorr, 1);
                    } else {
                        $this->cdreducont = $requisicao['CDREDUCONT'];
                        $this->dscontcorr = $requisicao['DSCONTCORR'];
                    }


                }
                //Verifica se o banco do extrato é o mesmo informado
                if($banco === trim(substr($line, 0, 3))) {
                    //Seta cdbanco a inserir o extrato
                    $this->cdbanco = trim(substr($line, 0, 3));
                } else {
                    throw new \Exception(self::BANCO_INV, 1);
                }

                //Data da movimentação
                $dia = trim(substr($line, 142,2));
                $mes = trim(substr($line, 144,2));
                $ano = trim(substr($line, 146,4));
                $dtMovreal = $dia.'/'.$mes.'/'.$ano;

                //Tipo
                $tipo = trim(substr($line, 7, 1)); //[1-Saldo Ini] [3-Lanc] [5-Saldo Fim]

                //Valor
                $vrLanc = floatval(trim(substr($line, 150, 16)).'.'.trim(substr($line, 166, 2)));

                //Credito ou debito
                $operacao = trim(substr($line, 168, 1)); // D-Debito / C-Credito

                //Verifica a categoria da operação
                if($tipo === '1') {
                    $this->saldoIni($vrLanc, $operacao, $dtMovreal);
                } else if ($tipo === '3') {
                    $nrdocmovreal = str_pad(trim(substr($line, 201, 20)), 20, '0', STR_PAD_LEFT);
                    $dshistmovreal = trim(substr($line, 176, 25));
                    $cdcatelanc = trim(substr($line, 169, 3));
                    $this->lanc($vrLanc, $operacao, $dtMovreal, $nrdocmovreal,
                                $dshistmovreal, $cdcatelanc);
                } else if ($tipo === '5') {
                    $this->saldoFim($vrLanc, $operacao, $dtMovreal);
                }
            } else {
                $this->cdbanco = '';
            }
        }
    }

    /**
     * Importação de extratos de conta corrente do Banco Santander
     * Conforme layout FEBRABAN 240 posições Versão 040
     *
     * Unica diferença entre esse modelo do moledo do Nossa Caixa, HSBC e Brasil
     * é a posição em que fica o código da agencia.
     *
     * @param  string $file  dados do extrato
     * @param  string $banco CDBANCCNAB do banco
     */
    public function readSantander($file, $banco) {
        $fileLines = $this->returnStringInlines($file);
        foreach($fileLines as $line) {
            //Desconsidera header e trailer de arquivo
            $checkLinha1 = trim(substr($line, 7, 1));
            $checkLinha2 = trim(substr($line, 7, 1));
            if(($checkLinha1 !== '0' && $checkLinha1 !== "") &&  ($checkLinha2 !== '9' && $checkLinha2 !== "")) {

                //Verifica se o arquivo é um arquivo de extrato para conciliação
                if(trim(substr($line, 7, 1)) === '1' && trim(substr($line, 7, 1)) === 'E') {
                    throw new \Exception(self::ARQUIVO_INV, 1);
                }

                //Executa apenas uma vez
                if(empty($this->cdbanco)) {
                    //Calcula o número de dígitos que compõe a conta corrente
                    $numberCarater = $this->calcNumberMask($banco);

                    //Seta a conta a inserir o extrato conforme validações
                    //Verifica se a conta corrente possuir dígito verificador
                    if(substr_count($this->maskContcorr, '-') > 0) {
                        //Determina a quantidade de dígitos verificadores da conta corrente
                        $numberVerificador = $this->calcNumberIndent($this->maskContcorr);
                        if($numberVerificador > 1) {
                            $this->cdcontcorr = substr(trim(substr($line, 58, 14)), ((-1)*$numberCarater));
                        } else {
                            $this->cdcontcorr = substr(trim(substr($line, 58, 13)), ((-1)*$numberCarater));
                        }
                    } else {
                        $this->cdcontcorr = substr(trim(substr($line, 58, 12)), ((-1)*$numberCarater));
                    }

                    //Verifica a existência da conta
                    $contaData = $this->Bco54200factory->getContCorrLike($banco, ltrim(substr($line, 52, 5),'0'), $this->cdcontcorr, $this->nrorg);

                    if(empty($contaData)) {
                        $agencia = ltrim(substr($line, 52, 5),'0');
                        throw new \Exception(self::CODREDU_NCAD . "<br>Agência: " . $agencia . "<br>Número da conta corrente: " . $this->cdcontcorr, 1);
                    } else {
                        $this->cdreducont = $contaData['CDREDUCONT'];
                        $this->dscontcorr = $contaData['DSCONTCORR'];
                    }

                    //Verifica se o banco do extrato é o mesmo informado
                    if($banco === trim(substr($line, 0, 3))) {
                        //Seta cdbanco a inserir o extrato
                        $this->cdbanco = trim(substr($line, 0, 3));
                    } else {
                        throw new \Exception(self::BANCO_INV, 1);
                    }
                }
                //Data da movimentação
                $dia = trim(substr($line, 142,2));
                $mes = trim(substr($line, 144,2));
                $ano = trim(substr($line, 146,4));
                $dtMovreal = $dia.'/'.$mes.'/'.$ano;

                //Tipo
                $tipo = trim(substr($line, 7, 1)); //[1-Saldo Ini] [3-Lanc] [5-Saldo Fim]

                //Valor
                $vrLanc = floatval(trim(substr($line, 150, 16)).'.'.trim(substr($line, 166, 2)));

                //Credito ou debito
                $operacao = trim(substr($line, 168, 1)); // D-Debito / C-Credito

                //Verifica a categoria da operação
                if(empty($operacao) && trim(substr($line, 169, 1)) === '1') {
                    $operacao = 'D';
                } else if(empty($operacao) && trim(substr($line, 169, 1)) === '2') {
                    $operacao = 'C';
                } else if (!empty($operacao)){
                    if($tipo === '1') {
                        $this->saldoIni($vrLanc, $operacao, $dtMovreal);
                    } else if ($tipo === '3') {
                        $nrdocmovreal = str_pad(trim(substr($line, 201, 20)), 20, '0', STR_PAD_LEFT);
                        $dshistmovreal = trim(substr($line, 176, 25));
                        $cdcatelanc = trim(substr($line, 169, 3));
                        $this->lanc($vrLanc, $operacao, $dtMovreal, $nrdocmovreal,
                                    $dshistmovreal, $cdcatelanc);
                    } else if ($tipo === '5') {
                        $this->saldoFim($vrLanc, $operacao, $dtMovreal);
                    }
                }
            } else {
                $this->cdbanco = '';
            }
        }
    }

    /**
     * Importação de extratos de conta corrente dos bancos:
     * NOSSA CAIXA 151
     * HSBC 399
     * BRASIL 001
     *
     * Unica diferença entre esse modelo do moledo do Santander é a posição em que fica
     * o código da agencia.
     *
     * @param  string $file  dados do extrato
     * @param  string $banco CDBANCCNAB do banco
     */
    public function readFebraban($file, $banco) {
        $fileLines = $this->returnStringInlines($file);
        foreach($fileLines as $line) {
            //Desconsidera header e trailer de arquivo
            $checkLinha1 = trim(substr($line, 7, 1));
            $checkLinha2 = trim(substr($line, 7, 1));
            if(($checkLinha1 !== '0' && $checkLinha1 !== "") &&  ($checkLinha2 !== '9' && $checkLinha2 !== "")) {

                //Verifica se o arquivo é um arquivo de extrato para conciliação
                if(trim(substr($line, 7, 1)) === '1' && trim(substr($line, 7, 1)) === 'E') {
                    throw new \Exception(self::ARQUIVO_INV, 1);
                }

                //Executa apenas uma vez
                if(empty($this->cdbanco)) {
                    //Calcula o número de dígitos que compõe a conta corrente
                    $numberCarater = $this->calcNumberMask($banco);

                    //Seta a conta a inserir o extrato conforme validações
                    //Verifica se a conta corrente possuir dígito verificador
                    if(substr_count($this->maskContcorr, '-') > 0) {
                        //Determina a quantidade de dígitos verificadores da conta corrente
                        $numberVerificador = $this->calcNumberIndent($this->maskContcorr);
                        if($numberVerificador > 1) {
                            $this->cdcontcorr = substr(trim(substr($line, 58, 14)), ((-1)*$numberCarater));
                        } else {
                            $this->cdcontcorr = substr(trim(substr($line, 58, 13)), ((-1)*$numberCarater));
                        }
                    } else {
                        $this->cdcontcorr = substr(trim(substr($line, 58, 12)), ((-1)*$numberCarater));
                    }

                    //Verifica a existência da conta
                    $contaData = $this->Bco54200factory->getContCorrLike($banco, trim(substr($line, 53, 4)), $this->cdcontcorr, $this->nrorg);
                    if(empty($contaData)) {
                        $agencia = trim(substr($line, 53, 4));
                        throw new \Exception(self::CODREDU_NCAD . "<br>Agência: " . $agencia . "<br>Número da conta corrente: " . $this->cdcontcorr, 1);
                    } else {
                        $this->cdreducont = $contaData['CDREDUCONT'];
                        $this->dscontcorr = $contaData['DSCONTCORR'];
                    }

                    //Verifica se o banco do extrato é o mesmo informado
                    if($banco === trim(substr($line, 0, 3))) {
                        //Seta cdbanco a inserir o extrato
                        $this->cdbanco = trim(substr($line, 0, 3));
                    } else {
                        throw new \Exception(self::BANCO_INV, 1);
                    }
                }
                //Data da movimentação
                $dia = trim(substr($line, 142,2));
                $mes = trim(substr($line, 144,2));
                $ano = trim(substr($line, 146,4));
                $dtMovreal = $dia.'/'.$mes.'/'.$ano;

                //Tipo
                $tipo = trim(substr($line, 7, 1)); //[1-Saldo Ini] [3-Lanc] [5-Saldo Fim]

                //Valor
                $vrLanc = floatval(trim(substr($line, 150, 16)).'.'.trim(substr($line, 166, 2)));

                //Credito ou debito
                $operacao = trim(substr($line, 168, 1)); // D-Debito / C-Credito

                //Verifica a categoria da operação
                if(empty($operacao) && trim(substr($line, 169, 1)) === '1') {
                    $operacao = 'D';
                } else if(empty($operacao) && trim(substr($line, 169, 1)) === '2') {
                    $operacao = 'C';
                } else if (!empty($operacao)){
                    if($tipo === '1') {
                        $this->saldoIni($vrLanc, $operacao, $dtMovreal);
                    } else if ($tipo === '3') {
                        $nrdocmovreal = str_pad(trim(substr($line, 201, 20)), 20, '0', STR_PAD_LEFT);
                        $dshistmovreal = trim(substr($line, 176, 25));
                        $cdcatelanc = trim(substr($line, 169, 3));
                        $this->lanc($vrLanc, $operacao, $dtMovreal, $nrdocmovreal,
                                    $dshistmovreal, $cdcatelanc);
                    } else if ($tipo === '5') {
                        $this->saldoFim($vrLanc, $operacao, $dtMovreal);
                    }
                }
            } else {
                $this->cdbanco = '';
            }
        }
    }

    /**
     * Remove Movimentos Existêntes no Sistema
     * @param $rows
     * @return array
     */
    public function removeMovimentosExistentesNoSistema($rows){
        $movimentos = array();
        
        foreach($rows as $row) {
            $nrdocmovreal = str_pad($row['NRDOCMOVREAL'], 20, '0', STR_PAD_LEFT);
            $dtmovreal    = self::formataData($row['DTMOVREAL']);
            $vrmovreal    = $row['VRMOVREAL'];
            
            $cdfitidextr  = isset($row['CDFITIDEXTR'])?$row['CDFITIDEXTR']:null;

            /** @var Movreal $movreal */
            $movreal = $this->Bco54200factory->getMovReal(
                $this->environment->getNrOrgTrab(), $row['CDREDUCONT'],
                $dtmovreal, $nrdocmovreal, $vrmovreal, $cdfitidextr
            );

            if(!$movreal)
                $movimentos[] = $row;
        }

        return $movimentos;
    }

    /**
     * Remove Movimentos Duplicados no Arquivo
     * @param $rows
     * @return array
     */
    public function removeMovimentosDuplicadosNoArquivo($rows){
        $movimentos = array();
        $movimentosDuplicados = array();
        foreach($rows as $row) {
            $nrdocmovreal = str_pad($row['NRDOCMOVREAL'], 20, '0', STR_PAD_LEFT);
            $dtmovreal    = self::formataData($row['DTMOVREAL']);
            $vrmovreal    = $row['VRMOVREAL'];
            $cdfitidextr  = isset($row['CDFITIDEXTR'])?$row['CDFITIDEXTR']:'';
            $identificadorMovimento = $nrdocmovreal.$dtmovreal.$vrmovreal.$cdfitidextr;
            
            if (array_key_exists($identificadorMovimento, $movimentos)){
                $movimentosDuplicados[] = $row;
            }else{
                $movimentos[$identificadorMovimento] = $row;
            }
        }
        return array($movimentos, $movimentosDuplicados);
    }

    /**
     * Formata Data
     * @param $data
     * @return \DateTime|string|\Zeedhi\Framework\ORM\DateTime
     */
    public function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    /**
     
     */
    public function buscaExtratoImportado($row){
        
        $paramConsulta = array(
         'CDREDUCONT' => '',
         'CDSEQUEXTR' => '',
         'NRSEQMOVREAL'=> ''
        );

        foreach ($row as $key => $value){
            if($value["name"] == 'CDREDUCONT'){
                $paramConsulta["CDREDUCONT"] = $value["value"];
            } 
            if($value["name"] == 'CDSEQUEXTR') {
                $paramConsulta["CDSEQUEXTR"] = $value["value"];
            } 
            if ($value["name"] == 'NRSEQMOVREAL'){
                 $paramConsulta["NRSEQMOVREAL"] = $value["value"];
            }
        }
        
        $retorno = $this->Bco54200factory->buscaExtratoImportado($paramConsulta);
        return $retorno;
    }
    
    public function getExtrato($row){
        if(array_key_exists('NRORG', $row)){
            $nrorg = $row['NRORG'];
        } else {
            $nrorg = $this->nrorg;
        }

        //'DSHISTMOVREAL' => substr($row['DSHISTMOVREAL'], 0, 40),
        $params = array(
         'CDREDUCONT' => $row['CDREDUCONT'],
         'CDSEQUEXTR' => $row['CDSEQUEXTR'],
         'NRDOCMOVREAL' => $row['NRDOCMOVREAL'],
         'DSHISTMOVREAL' => $row['DSHISTMOVREAL'],
         'DTMOVREAL'=> $row['DTMOVREAL'],
         'VRMOVREAL'=> $row['VRMOVREAL'],
         'IDMOVREAL'=> $row['IDMOVREAL'],
         'IDTPMOVREAL'=> $row['IDTPMOVREAL'],
         'CDFITIDEXTR'=> $row['CDFITIDEXTR'],
         'NRORG'=> $nrorg
        );

        $retorno = $this->Bco54200factory->getExtrato($params);
        return $retorno;
    }
}


